#import "MyObject.h"
#import "Person.h"
#import "Pair.h"

@implementation MyObject

#pragma mark important utilities

- (void) pair:(Person*)p1 with:(Person*)p2 {
	Pair* p = [[Pair alloc] init];
	[p setPerson1: p1];
	[p setPerson2: p2];
	[p1 setPair:p];
	[p2 setPair:p];
	[self updateInterface:self];	
}

- (BOOL) existsPersonWithName: (NSString*) n {
	int i,u = [persons count];
	for (i=0;i<u;i++) {
		Person* p = [persons objectAtIndex:i];
		if ([[p name] caseInsensitiveCompare: n] == NSOrderedSame)
			// case insensitive to match how AppleScript "exists" works
			return YES;
	}
	return NO;
}


#pragma mark interface routines
// === ignore these; our application has no interface
// === (it did once, but that was just for testing purposes early on)

- (void) awakeFromNib; 
{
	persons = [[NSMutableArray array] retain];
	Person* p = [[Person alloc] init];
	[p setName:@"Jack"];
	[p setGender:MALE];
	[p setMaster:self];
	[persons addObject: p];
	p = [[Person alloc] init];
	[p setName:@"Jill"];
	[p setGender:FEMALE];
	[p setMaster:self];
	[persons addObject: p];	
	pairs = [[NSMutableArray array] retain];
	[self updateInterface:self];
}

- (void) updateInterface:(id)sender; 
{
	[malePopup removeAllItems];
	[femalePopup removeAllItems];
	[malePopup addItemWithTitle: @"Males:"];
	[femalePopup addItemWithTitle: @"Females:"];
	NSMenu* maleMenu = [malePopup menu];
	NSMenu* femaleMenu = [femalePopup menu];
	[maleMenu setAutoenablesItems: NO];
	[femaleMenu setAutoenablesItems:NO];
	int i,u = [persons count];
	for (i=0;i<u;i++) {
		NSMenuItem* mi = [[NSMenuItem alloc] init];
		Person* p = [persons objectAtIndex:i];
		NSString* n = [p name];
		[mi setTitle: n];
		[mi setRepresentedObject: p];
		NSMenu* m = ([p gender] == MALE ? maleMenu : femaleMenu);
		[m addItem:mi];
		[mi setEnabled: ([p pair] == nil)];
	}
}


- (IBAction)doMakePair:(id)sender;
{
	int whichMale = [malePopup indexOfSelectedItem];
	int whichFemale = [femalePopup indexOfSelectedItem];
	if (whichMale == 0 || whichFemale == 0) {
		NSBeep();
		return;
	}
	[self pair:[[malePopup selectedItem] representedObject]
		  with:[[femalePopup selectedItem] representedObject]];
}

- (IBAction)doMakePerson:(id)sender;
{
	NSString* newName = [nameField stringValue];
	if (!newName || [newName isEqualToString: @""] || [self existsPersonWithName:newName]) {
		NSBeep();
		return;
	}
	int gender = [genderMatrix selectedRow];
	Person* p = [[Person alloc] init];
	[p setName: newName];
	[p setGender: gender == 0 ? MALE : FEMALE];
	[p setMaster: self];
	[persons addObject:p];
	[self updateInterface:self];
}


@end
